// testCopy.cxx

#include "testap3.h"

RECT  grcClip = {0};
//POINT ptClipSize;
HWND  hWndClip;

void Do_WM_RENDERALLFORMATS( HWND hWnd )
{
   if( !OpenClipboard( hWnd ) )
      return;

   EmptyClipboard();
   SendMessage( hWnd, WM_RENDERFORMAT, CF_DIB,     0L );
   SendMessage( hWnd, WM_RENDERFORMAT, CF_BITMAP,  0L );
   SendMessage( hWnd, WM_RENDERFORMAT, CF_PALETTE, 0L );
   CloseClipboard ();
}

int ColorsInPalette( HPALETTE hPal )
{
	int nColors;

	if( !hPal )
		return 0;

	GetObject( hPal, sizeof( nColors ), (LPSTR) &nColors );

	return nColors;
}

#define DONT_CHANGE_FLAGS -1

// The following is the palette version that goes in a
//  LOGPALETTE's palVersion field.
#define PALVERSION   0x300

HPALETTE CopyPaletteChangingFlags (HPALETTE hPal, BYTE bNewFlag)
{
	WORD			wEntries, i;
	HANDLE			hLogPal;
	LPLOGPALETTE	lpLogPal;

	if( !hPal )
		return NULL;

	wEntries = ColorsInPalette( hPal );

	if( !wEntries )
		return NULL;

	hLogPal = MALLOC( sizeof (LOGPALETTE) + (sizeof (PALETTEENTRY) * wEntries) );

	if( !hLogPal )
		return NULL;

   lpLogPal = (LPLOGPALETTE)hLogPal;
	lpLogPal->palVersion    = PALVERSION;
	lpLogPal->palNumEntries = wEntries;

	GetPaletteEntries( hPal, 0, wEntries, lpLogPal->palPalEntry );

	if( bNewFlag != DONT_CHANGE_FLAGS )
	{
		for( i = 0;  i < wEntries;  i++ )
		{
			lpLogPal->palPalEntry[i].peFlags = bNewFlag;
		}
	}

	hPal = CreatePalette( lpLogPal );

	MFREE(hLogPal);

	return hPal;
}

HBITMAP CopyScreenToBitmap(LPRECT lpRect)
{
	HDC hScrDC, hMemDC;           // screen DC and memory DC
	HBITMAP hBitmap, hOldBitmap;  // handles to device-dependent bitmaps
	int nX, nY, nX2, nY2;         // coordinates of rectangle to grab
	int nWidth, nHeight;          // DIB width and height
	int xScrn, yScrn;             // screen resolution

	// check for an empty rectangle
	if( IsRectEmpty( lpRect ) )
		return NULL;

	// create a DC for the screen and create
	// a memory DC compatible to screen DC
	hScrDC = CreateDC( L"DISPLAY", NULL, NULL, NULL );
	if( !hScrDC )	// If no memory DC
		return NULL;

	hMemDC = CreateCompatibleDC( hScrDC );
	if( !hMemDC )
	{
		// No compatible DC
		DeleteDC( hScrDC );
		return NULL;
	}

	// get points of rectangle to grab
	nX  = lpRect->left;
	nY  = lpRect->top;
	nX2 = lpRect->right;
	nY2 = lpRect->bottom;

	// get screen resolution
	xScrn = GetDeviceCaps( hScrDC, HORZRES );
	yScrn = GetDeviceCaps( hScrDC, VERTRES );

	// make sure bitmap rectangle is visible
	if( nX < 0 )
		nX = 0;
	if( nY < 0 )
		nY = 0;

	if( nX2 > xScrn )
		nX2 = xScrn;
	if( nY2 > yScrn )
		nY2 = yScrn;

	nWidth  = nX2 - nX;
	nHeight = nY2 - nY;

	// create a bitmap compatible with the screen DC
	hBitmap = CreateCompatibleBitmap( hScrDC, nWidth, nHeight );
	if( !hBitmap )
	{
		DeleteDC( hScrDC );
		DeleteDC( hMemDC );
		return NULL;
	}

	// select new bitmap into memory DC
	hOldBitmap = (HBITMAP)SelectObject( hMemDC, hBitmap );

	// bitblt screen DC to memory DC
	BitBlt( hMemDC,	// Destination
		0, 0,
		nWidth, nHeight,	// Size
		hScrDC,				// Source
		nX, nY,				// from location
		SRCCOPY );			// COPY operation

	// select old bitmap back into memory DC and get handle to
	// bitmap of the screen
	hBitmap = (HBITMAP)SelectObject( hMemDC, hOldBitmap );

	// clean up
	DeleteDC( hScrDC );
	DeleteDC( hMemDC );

	// return handle to the bitmap
	return hBitmap;

}

void Get_Screen_Coordinates( HWND hWnd, PRECT prc, BOOL IsClient )
{
   POINT ptClientUL, ptClientLR;
   // GetClientRect(hwndMain, &rcClient); 
   // extract RECT to UL/LR points
   ptClientUL.x = prc->left; 
   ptClientUL.y = prc->top; 
   ptClientLR.x = prc->right;
   ptClientLR.y = prc->bottom; 
   if( IsClient ) {
      // Add one to the right and bottom sides, because the 
      // coordinates retrieved by GetClientRect do not 
      // include the far left and lowermost pixels. 
      ptClientLR.x += 1;
      ptClientLR.y += 1; 
   }

   ClientToScreen(hWnd, &ptClientUL);
   ClientToScreen(hWnd, &ptClientLR);

   sprtf( "Converted Window (%d,%d,%d,%d) to screen (%d,%d,%d,%d) (%s)\n",
      prc->left, prc->top, prc->right, prc->bottom,
      ptClientUL.x, ptClientUL.y, ptClientLR.x, ptClientLR.y,
      ( IsClient ? "IsClient" : "" ) );

   // return adjusted coordinates
   prc->left   = ptClientUL.x; 
   prc->top    = ptClientUL.y;
   prc->right  = ptClientLR.x;
   prc->bottom = ptClientLR.y;
   // ===========================
}

// copy screen rectangle to a bitmap
HBITMAP ScreenRect2Bitmap( HWND hWnd, PRECT prc )
{
   HBITMAP hBitmap = NULL;
   RECT  rc = *prc;
   if( !IsRectEmpty( &rc ) ) {
      Get_Screen_Coordinates( hWnd, &rc, FALSE );
      hBitmap = CopyScreenToBitmap( &rc );  // copy this part of SCREEN
   }
   return hBitmap;
}

// HANDLE RenderFormat( HWND hWndClip, int cf, POINT ptDIBSize_NOT_USED )
HANDLE RenderFormat( HWND hWnd, int cf )
{
	HBITMAP   hBitmap;
	HPALETTE  hPalette;        // Handle to the bitmap's palette.
	HANDLE    h = NULL;		// Start with NOTHING

	hPalette     = NULL;
	hBitmap      = m_hBitmap;
	switch( cf )
	{
	case CF_BITMAP:
      if( m_hBitmap ) {
         sprtf( "RenderFormat: CF_BITMAP, with loaded BMP\n" );
         h = CropBitmap( hBitmap, hPalette, &grcClip, NULL );
      } else {
         // NO BITMAP - get clip, or FULL current window display
         RECT  rcc;
         RECT rc = m_Clip; // COPY CLIP region
         BOOL  bIsClient = FALSE;   // assume using CLIP

         sprtf( "RenderFormat: CF_BITMAP = no BMP loaded\n" );
         if( IsRectEmpty( &rc ) ) {
            GetClientRect( hWnd, &rc );
            bIsClient = TRUE;
         }
         rcc = rc;
         Get_Screen_Coordinates( hWnd, &rcc, bIsClient );
         hBitmap = CopyScreenToBitmap( &rcc );  // copy this part of SCREEN
         if( hBitmap ) {
            sprtf( "RenderFormat: Returning HANDLE %p ...\n", hBitmap );
            h = hBitmap;
         } else {
            sprtf( "RenderFormat: FAILED TO GET HANDLE!\n" );
         }
      }
		break;

	case CF_DIB:
		{
			HBITMAP hbm;
         // NOTE:  For simplicity, we use the display device to crop
			//        the bitmap.  This means that we may lose color
			//        precision (if the display device has less color
			//        precision than the DIB).  This isn't usually a
			//        problem, as users shouldn't really be editting
			//        images on devices that can't display them.
			//hbm = (HBITMAP)RenderFormat( hWndClip, CF_BITMAP, ptDIBSize );
         sprtf( "RenderFormat: CF_DIB - use CF_BITMAP ...\n" );
			hbm = (HBITMAP)RenderFormat( hWnd, CF_BITMAP );
			if( hbm ) {
				h = DVBitmapToDIB( hbm, hPalette );
				DeleteObject( hbm );
			}
         sprtf( "RenderFormat: CF_DIB - returning %p ...\n", h );
			break;
		}

	case CF_PALETTE:
      sprtf( "RenderFormat: CF_PALETTE\n" );
		if( hPalette )
			h = CopyPaletteChangingFlags( hPalette, 0 );
		break;
   default:
      sprtf( "RenderFormat: uncased: (%d)\n", cf  );
      break;
	}

	return h;

}	// End - HANDLE RenderFormat( HWND hWnd, int cf )


void Do_WM_RENDERFORMAT( HWND hWnd, WPARAM wParam )
{
   HANDLE hClipBoardData;
   hClipBoardData = RenderFormat( hWnd, (int)wParam );
   if( hClipBoardData ) {
      SetClipboardData( (UINT)wParam, hClipBoardData );
      sprtf( "WM_RENDERFORMAT: Set CLIPBOARD to %p ...(%d)\n", hClipBoardData, wParam);
   } else {
      sprtf( "WM_RENDERFORMAT: FAILED (%d)\n", wParam);
   }
}


//HWND  HandleCopyClipboard( void )
void Do_IDM_EDITCOPY( HWND hWnd )
{
   // Clean clipboard of contents, and tell it we can render
	//  a DIB, a DDB, and/or a palette.
	if( OpenClipboard( hWnd ) ) {
      EmptyClipboard();
      SetClipboardData( CF_DIB,     NULL );
      SetClipboardData( CF_BITMAP,  NULL );
      SetClipboardData( CF_PALETTE, NULL );
      CloseClipboard ();
      // Set our globals to tell our app which child window
		//  owns the clipboard, and the clipping rectangle at
		//  the time of the copy.  If the clipping rectangle is
		//  empty, then use the entire DIB window.
      if( m_hDIB && m_hBitmap ) {
		   grcClip       = m_Clip;
		   //ptClipSize.x = m_sDIBSize.cx;
         //ptClipSize.y = m_sDIBSize.cy;
         hWndClip      = hWnd;
		   if( IsRectEmpty( &grcClip ) )
		   {
			   grcClip.left   = 0;
			   grcClip.top    = 0;
			   grcClip.right  = m_sDIBSize.cx; //ptClipSize.x;
			   grcClip.bottom = m_sDIBSize.cy; //ptClipSize.y;
		   }
         sprtf( "Done IDM_EDITCOPY, clip = %dx%d\n", grcClip.right, grcClip.bottom );
            // ptClipSize.x, ptClipSize.y );
      } else {
         GetWindowRect( hWnd, &grcClip );
		   //ptClipSize.x = grcClip.right;
         //ptClipSize.y = grcClip.bottom;
         sprtf( "Done IDM_EDITCOPY, window = %dx%d\n", grcClip.right, grcClip.bottom );
            // ptClipSize.x, ptClipSize.y );
      }

   }
	else
   {
      // DIBError( ERR_CLIPBUSY );
   }
}

// eof - testCopy.cxx
