// testGDI.cxx
// collection of Graphic Device Interface functions

#include "testap3.h"

void PaintRectOutline( HDC hdc, PRECT prc )
{
   POINT aptRect[5];
   aptRect[0].x = prc->left;
   aptRect[0].y = prc->top;
   aptRect[1].x = prc->right;
   aptRect[1].y = prc->top;
   aptRect[2].x = prc->right;
   aptRect[2].y = prc->bottom;
   aptRect[3].x = prc->left;
   aptRect[3].y = prc->bottom;
   aptRect[4].x = prc->left;
   aptRect[4].y = prc->top;
   Polyline(hdc, aptRect, 5);
}

void DrawColoredRect( HDC hdc, PRECT prc, COLORREF cr )
{
   HPEN hp, old;
   hp = CreatePen( PS_SOLID, 1, cr );
   if(hp) {
      old = (HPEN)SelectObject(hdc, hp);
      // this has the problem that it FILLS the rectangle
      //Rectangle( hdc, prc->left, prc->top, prc->right, prc->bottom );
      PaintRectOutline( hdc, prc ); // this does ONLY a PolyLine draw
      hp = (HPEN)SelectObject(hdc, old);
      DeleteObject(hp);
   }
}

void DrawColoredEllipse( HDC hdc, PRECT prc, COLORREF cr )
{
   HPEN hp, old;
   hp = CreatePen( PS_SOLID, 1, cr );
   if(hp) {
      old = (HPEN)SelectObject(hdc, hp);
      // this has the problem that it FILLS the circle
      // BUT MAYBE IF I USE A HOLLWO BRUSH
      HBRUSH hbr, hbrold;
      hbr = GetHollowBrush();
      if(hbr) {
         hbrold = (HBRUSH)SelectObject(hdc, hbr);
         Ellipse( hdc, prc->left, prc->top, prc->right, prc->bottom );
         hbr = (HBRUSH)SelectObject(hdc,hbrold);
         DeleteObject(hbr);
      } else {
         // so have written my OWN outlining ...
         DrawEllipseOutline( hdc,
            (prc->left + prc->right) / 2,
            (prc->top + prc->bottom) / 2,
            RECTWIDTH(prc) / 2 );
      }

      hp = (HPEN)SelectObject(hdc, old);
      DeleteObject(hp);
   }
}


LPRGNDATA GetRgnDataBlock( HRGN rgn )
{
   DWORD sz = GetRegionData(rgn,0,NULL);
   BYTE * cp = MALLOC(sz);
   LPRGNDATA prd = (LPRGNDATA)cp;
   if(prd) {
      GetRegionData(rgn, sz, prd);
   }
   return prd;
}

/* ============================
typedef struct _RGNDATA { 
    RGNDATAHEADER rdh; 
    char          Buffer[1]; 
} RGNDATA, *PRGNDATA; 
typedef struct _RGNDATAHEADER { 
    DWORD dwSize; 
    DWORD iType; 
    DWORD nCount; 
    DWORD nRgnSize; 
    RECT  rcBound; 
} RGNDATAHEADER, *PRGNDATAHEADER; 
Setting a REGION to 120,120, through
HRGN  rgn1 = CreateEllipticRgn(0,0,120,120);
creates a region with 71 rectangles, as follows
Note this describes the POINTS on an ELIPSE
14:26:40 - Rgn bounding rect = (0,0,119,119), count 71
(57,0,62,1) (47,1,72,2) (44,2,75,3) (41,3,78,4) (38,4,81,5) 
(35,5,84,6) (34,6,85,7) (32,7,87,8) (30,8,89,9) (28,9,91,10) 
(26,10,93,11) (25,11,94,12) (24,12,95,13) (23,13,96,14) (21,14,98,15) 
(20,15,99,16) (19,16,100,17) (18,17,101,18) (17,18,102,19) (16,19,103,20) 
(15,20,104,21) (14,21,105,23) (13,23,106,24) (12,24,107,25) (11,25,108,26) 
(10,26,109,28) (9,28,110,30) (8,30,111,32) (7,32,112,33) (6,33,113,35) 
(5,35,114,38) (4,38,115,41) (3,41,116,44) (2,44,117,47) (1,47,118,57) 
(0,57,119,62) (1,62,118,72) (2,72,117,75) (3,75,116,78) (4,78,115,81) 
(5,81,114,84) (6,84,113,86) (7,86,112,87) (8,87,111,89) (9,89,110,91) 
(10,91,109,93) (11,93,108,94) (12,94,107,95) (13,95,106,96) (14,96,105,98) 
(15,98,104,99) (16,99,103,100) (17,100,102,101) (18,101,101,102) (19,102,100,103) 
(20,103,99,104) (21,104,98,105) (23,105,96,106) (24,106,95,107) (25,107,94,108) 
(26,108,93,109) (28,109,91,110) (30,110,89,111) (32,111,87,112) (34,112,85,113) 
(35,113,84,114) (38,114,81,115) (41,115,78,116) (44,116,75,117) (47,117,72,118) 
(57,118,62,119) 

   ============================= */
void OutputRgn( HDC hdc, HRGN rgn )
{
   LPRGNDATA prd = GetRgnDataBlock(rgn);
   PRECT prc;
   DWORD i, wrap;
   int   oldt;
   if(!prd)
      return;
   sprtf( "Rgn bounding rect = %s, count %d\n", GetRectStg( &prd->rdh.rcBound ),
      prd->rdh.nCount );
   prc = (PRECT)&prd->Buffer[0];
   wrap = 0;
   oldt = add_sys_time(0);
   for(i = 0; i < prd->rdh.nCount; i++) {

      sprtf("%s ", GetRectStg(prc));
      wrap++;
      if( wrap > 4 ) {
         sprtf("\n");
         wrap = 0;
      }
      PaintRectOutline(hdc, prc);
      prc++;
   }
   if(wrap)
      sprtf("\n");
   add_sys_time(oldt);
   MFREE(prd);
}

/* ==============================
16:07:41 - Rgn bounding rect = (0,0,119,119), count 71 (Center 80,80, radius 60)
First set of 71 + 1 points...
(59,0) first point...
(57,0) (47,1) (44,2) (41,3) (38,4) (35,5) (34,6) (32,7) (30,8) (28,9) (26,10) 
(25,11) (24,12) (23,13) (21,14) (20,15) (19,16) (18,17) (17,18) (16,19) (15,20) (14,21) 
(13,23) (12,24) (11,25) (10,26) (9,28) (8,30) (7,32) (6,33) (5,35) (4,38) (3,41) 
(2,44) (1,47)  to lower half
(0,62) (1,72) (2,75) (3,78) (4,81) (5,84) (6,86) (7,87) (8,89) (9,91) (10,93) 
(11,94) (12,95) (13,96) (14,98) (15,99) (16,100) (17,101) (18,102) (19,103) (20,104) (21,105) 
(23,106) (24,107) (25,108) (26,109) (28,110) (30,111) (32,112) (34,113) (35,114) (38,115) (41,116) 
(44,117) (47,118) (57,119) 
Second set of 71 + 1 points...
(62,119) (72,118) (75,117) (78,116) (81,115) (84,114) (85,113) (87,112) (89,111) (91,110) (93,109) 
(94,108) (95,107) (96,106) (98,105) (99,104) (100,103) (101,102) (102,101) (103,100) (104,99) (105,98) 
(106,96) (107,95) (108,94) (109,93) (110,91) (111,89) (112,87) (113,86) (114,84) (115,81) (116,78) 
(117,75) (118,72)  to upper half
(119,57) (118,47) (117,44) (116,41) (115,38) (114,35) (113,33) (112,32) (111,30) (110,28) (109,26) 
(108,25) (107,24) (106,23) (105,21) (104,20) (103,19) (102,18) (101,17) (100,16) (99,15) (98,14) 
(96,13) (95,12) (94,11) (93,10) (91,9) (89,8) (87,7) (85,6) (84,5) (81,4) (78,3) 
(75,2) (72,1) (62,0) 
(59,0) last = first
PAINT 144 points - just join lines ...(144)
   ============================== */
#define  dbg_sprtf

void DrawEllipseOutline( HDC hdc, int centx, int centy, int radius )
{
   int diam = radius * 2;
   HRGN rgn = CreateEllipticRgn(0,0,diam,diam);
   if(rgn) {
      LPRGNDATA prd = GetRgnDataBlock(rgn);
      if(prd) {
         DWORD max = prd->rdh.nCount;
         DWORD maxps = sizeof(POINT) * ((max * 2) + 2);
         PPOINT pp = (PPOINT)MALLOC( maxps );
         PRECT prc;
         dbg_sprtf( "Rgn bounding rect = %s, count %d (Center %d,%d, radius %d)\n",
            GetRectStg( &prd->rdh.rcBound ),
            max, centx, centy, radius );
         prc = (PRECT)&prd->Buffer[0]; // point to FIRST rectangle
         if(pp) {
            DWORD i, k, wrap;
            int cx, cy, dt;
            int oldt = add_sys_time(0);
            POINT pt;
            DWORD hm = (max / 2);
            k = 0;
            wrap = 0;
            SecureZeroMemory(pp, maxps);
            dbg_sprtf( "First set of %d + 1 points...\n", max );
            dt = 0;
            for( i = 0; i < max; i++ ) {
               if( i == 0 ) {
                  cx = (prc->left + prc->right) / 2;
                  cy = prc->top - 1;
                  if(cy < 0)
                     cy++;
                  pp[k].x = cx;
                  pp[k].y = cy;
                  dbg_sprtf( "%s first point...\n", GetPointStg( &pp[k] ));
                  k++;
               }
               cx = prc->left;
               if( i < hm )
                  cy = prc->top;
               else {
                  if( !dt ) {
                     dbg_sprtf(" to lower half\n");
                     wrap = 0;
                     dt = 1;
                  }
                  cy = prc->bottom;
               }
               pp[k].x = cx;
               pp[k].y = cy;
               dbg_sprtf( "%s ", GetPointStg( &pp[k] ));
               k++;
               wrap++;
               if(wrap > 10) {
                  dbg_sprtf("\n");
                  wrap = 0;
               }
               prc++;
            }
            if(wrap) {
               dbg_sprtf("\n");
               wrap = 0;
            }
            dbg_sprtf( "Second set of %d + 1 points...\n", max );
            dt = 0;
            for( i = 0; i < max; i++ ) {
               prc--;
               cx = prc->right;
               if( i < hm )
                  cy = prc->bottom;
               else {
                  if( !dt ) {
                     dbg_sprtf(" to upper half\n");
                     wrap = 0;
                     dt = 1;
                  }
                  cy = prc->top;
               }
               pp[k].x = cx;
               pp[k].y = cy;
               dbg_sprtf( "%s ", GetPointStg( &pp[k] ));
               k++;
               wrap++;
               if(wrap > 10) {
                  dbg_sprtf("\n");
                  wrap = 0;
               }
            }

            if(wrap)
               dbg_sprtf( "\n" );
            // add closing POINT - back to FIRST
            pp[k].x = pp[0].x;
            pp[k].y = pp[0].y;
            dbg_sprtf( "%s last = first\n", GetPointStg( &pp[k] ));
            k++;
            // ========================================================
            dbg_sprtf( "PAINT %d points - just join lines ...(%d)\n",
               k,
               maxps / sizeof(POINT) );
            // set viewport origin to top, left ... CENTER - RADIUS
            SetViewportOrgEx(hdc, centx - radius, centy - radius, &pt); 
            Polyline(hdc, pp, k);   // paint the LINE, in the current PEN
            SetViewportOrgEx(hdc, pt.x, pt.y, NULL); // restore viewport origin 
            // ========================================================
            add_sys_time(oldt);
            MFREE(pp);
         }
         MFREE(prd);
      }
      DeleteObject(rgn);
   }
}

HBRUSH GetHollowBrush( void )
{
   LOGBRUSH lb;
   lb.lbStyle = BS_HOLLOW;
   lb.lbColor = RGB(0,0,0);   // ignored for HOLLOW
   lb.lbHatch = 0;            // ignored for HOLLOW
   HBRUSH hbr = CreateBrushIndirect( &lb );
   return hbr;
}

// eof - testGDI.cxx
