// testStamp.cxx

#include "testap3.h"

TmStamp::TmStamp( void )
{
#ifdef WIN32
   bGotHPT = QueryPerformanceFrequency( &freq ); // signal have high performance timer
   if(bGotHPT) {
      bGotCnt = QueryPerformanceCounter( &count ); // LARGE_INTEGER *lpPerformanceCount
      if( !bGotCnt )
         count.QuadPart = GetTickCount();
   }
   else
      count.QuadPart = GetTickCount();
#endif   // WIN32
   stamp( &init );
}

TmStamp::~TmStamp( void )
{
}

void TmStamp::stamp( PTMSTAMP pts )
{
   unsigned long t = timeGetTime();
   pts->secs = t / 1000;
   pts->usec = ( t - ( pts->secs * 1000 ) ) * 1000;
}

// difference between time stamps in microseconds (usec)
long TmStamp::timediff( PTMSTAMP a, PTMSTAMP b)
{
    return (1000000 * (a->secs - b->secs)) + (a->usec - b->usec);
}

// elapsed - in usec (microseconds)
long TmStamp::elapsed( PTMSTAMP pts )
{
   return timediff(pts, &init);
}

long TmStamp::elapsed( void )
{
#ifdef WIN32
   LARGE_INTEGER li, diff;
   long lg = 0;
   if( bGotHPT && bGotCnt ) {
      QueryPerformanceCounter( &li );
      diff.QuadPart = li.QuadPart - count.QuadPart;
      lg  = (long)((diff.QuadPart * 1000000) / freq.QuadPart);
   } else {
      li.QuadPart = GetTickCount();
      diff.QuadPart = li.QuadPart - count.QuadPart;
      lg  = (long)(diff.QuadPart * 1000);
   }
   return lg;
#else // !WIN32
   TMSTAMP ts;
   stamp( &ts );
   return timediff( &ts, &init );
#endif // WIN32 y/n
}

#ifdef WIN32
void TmStamp::get_freq( LARGE_INTEGER * pli )
{
   pli->QuadPart = freq.QuadPart;
}
#endif   // WIN32

void TmStamp::set_secs_struct( long usecs, PTMSTAMP pts )
{
   pts->secs = usecs / 1000000;
   pts->usec = ( usecs - ( pts->secs * 1000000 ) );
}

double TmStamp::get_secs_d( long usecs )
{
   return ((double)usecs / 1000000.0);
}

// eof - testStamp.cxx
